package Tapper::CLI;
# git description: v5.0.5-5-g33887a9

our $AUTHORITY = 'cpan:TAPPER';
# ABSTRACT: Tapper - tapper-* commandline tools
$Tapper::CLI::VERSION = '5.0.6';
use warnings;
use strict;

1; # End of Tapper::CLI

__END__

=pod

=encoding UTF-8

=head1 NAME

Tapper::CLI - Tapper - tapper-* commandline tools

=head1 AUTHOR

AMD OSRC Tapper Team <tapper@amd64.org>

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2020 by Advanced Micro Devices, Inc..

This is free software, licensed under:

  The (two-clause) FreeBSD License

=cut
